# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Package for caching of functions that take states or history browsers.'''

from .decorators import state_cache, history_cache