# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Persistent class and some related classes.

See documentation of Persistent and CrossProcessPersistent for more info.
'''

from .persistent import Persistent
from .cross_process_persistent import CrossProcessPersistent
from .personality import Personality
from .copy_modes import DontCopyPersistent
