# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This package defines a class ReadWriteLock in the module read_write_lock.
See that class's documentation.
'''
from .read_write_lock import *
