# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This package defines the SimpackGrokker and the Settings classes.

See their documentation for more info.
'''

from .settings import Settings
from .simpack_grokker import SimpackGrokker
